import { Component, ViewChild, forwardRef, Attribute, NgModule } from '@angular/core';
import { NG_VALUE_ACCESSOR, NG_VALIDATORS, FormsModule } from '@angular/forms';
import { CommonModule } from '@angular/common';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class MarkdownEditorComponent {
    /**
     * @param {?=} required
     * @param {?=} maxlength
     */
    constructor(required = false, maxlength = -1) {
        this.required = required;
        this.maxlength = maxlength;
        /**
         * flag for fullscreen mode
         */
        this.fullscreenEnabled = false;
        /**
         * flag for show/hide preview area
         */
        this.previewEnabled = true;
        /**
         * flag for disabling the editor (and all its controls)
         */
        this.disabled = false;
        this.onChange = (/**
         * @param {?} newValue
         * @return {?}
         */
        (newValue) => { });
        this.onTouched = (/**
         * @return {?}
         */
        () => { });
    }
    /**
     * @return {?}
     */
    get value() {
        return this._value || '';
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set value(value) {
        this._value = value;
        this.parsedHtml = marked(this._value);
        this.onChange(value);
    }
    /**
     * @return {?}
     */
    ngOnInit() {
    }
    /**
     * @return {?}
     */
    ngAfterViewInit() {
        this.codeMirror = CodeMirror.fromTextArea(this.editor.nativeElement, {
            mode: 'markdown',
            lineNumbers: true,
            autoCloseTags: true,
            lineWrapping: true,
            extraKeys: { 'Ctrl-Space': 'autocomplete' }
        });
        this.codeMirror.doc.on('change', (/**
         * @param {?} instance
         * @param {?} change
         * @return {?}
         */
        (instance, change) => {
            this.value = instance.getValue();
        }));
        this.resizeCodeMirror();
    }
    /* ControlValueAccessor methods  */
    /**
     * @param {?} obj
     * @return {?}
     */
    writeValue(obj) {
        this._value = '';
        if (typeof obj === 'string' && this.codeMirror) {
            this.codeMirror.setValue(obj);
            this.onChange(obj);
        }
    }
    /**
     * @param {?} fn
     * @return {?}
     */
    registerOnChange(fn) {
        this.onChange = fn;
    }
    /**
     * @param {?} fn
     * @return {?}
     */
    registerOnTouched(fn) {
        this.onTouched = fn;
    }
    /**
     * @param {?} isDisabled
     * @return {?}
     */
    setDisabledState(isDisabled) {
        this.disabled = isDisabled;
        if (this.codeMirror) {
            this.codeMirror.setOption('readOnly', isDisabled);
        }
    }
    /* END ControlValueAccessor methods  */
    /**
     * Validate implementation
     *
     * only required and maxlength for now
     * @param {?} control
     * @return {?}
     */
    validate(control) {
        /** @type {?} */
        let result = null;
        if (this.required && this._value.length === 0) {
            result = { required: true };
        }
        if (this.maxlength > 0 && this.value.length > this.maxlength) {
            result = { maxlength: true };
        }
        return result;
    }
    /**
     * Resizes the code mirror instance to the size of the container
     *
     * Used when the code mirror editor element size is changed (window resize, )
     * @return {?}
     */
    resizeCodeMirror() {
        setTimeout((/**
         * @return {?}
         */
        () => {
            /** @type {?} */
            const editorContainer = this.editor.nativeElement.parentNode;
            this.codeMirror.setSize(editorContainer.offsetWidth, editorContainer.offsetHeight);
        }), 200);
    }
    /**
     * @return {?}
     */
    toggleFullscreen() {
        this.fullscreenEnabled = !this.fullscreenEnabled;
        this.resizeCodeMirror();
    }
    /**
     * @return {?}
     */
    togglePreview() {
        this.previewEnabled = !this.previewEnabled;
        this.resizeCodeMirror();
    }
    /**
     * Inserts special content into the editor
     *
     * can be used for different buttons like bold, italic,
     *
     * Override the method if you want to customize the behaviour of the
     * insertion. For example if you have an own image selector and so on.
     *
     * @param {?} type type of the content that is to be inserted
     * @param {?=} value optional value (for special custom content)
     * @return {?}
     */
    insertContent(type, value) {
        /** @type {?} */
        let from = this.codeMirror.getCursor('from');
        // if value is set, use it, if not get the selection instead
        /** @type {?} */
        var selection = value ? value : this.codeMirror.getSelection();
        /** @type {?} */
        let to = this.codeMirror.getCursor('to');
        /** @type {?} */
        let newCursorPosition = { line: to.line, ch: to.ch };
        /** @type {?} */
        let additionalOffset = 0;
        /** @type {?} */
        var replacement = '';
        switch (type) {
            case 'bold':
                replacement = `**${selection}**`;
                this.codeMirror.replaceRange(replacement, from, to);
                additionalOffset = 2;
                break;
            case 'italic':
                replacement = `*${selection}*`;
                this.codeMirror.replaceRange(replacement, from, to);
                additionalOffset = 1;
                break;
            case 'quote':
                replacement = `> ${this.codeMirror.getLine(from.line)}`;
                this.codeMirror.replaceRange(replacement, { line: from.line, ch: 0 }, { line: from.line, ch: replacement.length });
                break;
            case 'list':
                replacement = `- ${this.codeMirror.getLine(from.line)}`;
                this.codeMirror.replaceRange(replacement, { line: from.line, ch: 0 }, { line: from.line, ch: replacement.length });
                break;
            case 'link':
                replacement = `[](${selection})`;
                this.codeMirror.replaceRange(replacement, from, to);
                additionalOffset = 1;
                break;
            case 'image':
                replacement = `![](${selection})`;
                this.codeMirror.replaceRange(replacement, from, to);
                additionalOffset = 2;
                break;
            case 'code':
                replacement = `\`\`\`\r\n${selection}\r\n\`\`\``;
                this.codeMirror.replaceRange(replacement, from, to);
                additionalOffset = 4;
                break;
            default:
        }
        newCursorPosition.ch += replacement.length - selection.length - additionalOffset;
        this.codeMirror.setCursor(newCursorPosition);
        this.codeMirror.focus();
        this.onChange(this._value);
    }
}
MarkdownEditorComponent.decorators = [
    { type: Component, args: [{
                selector: 'maddob-markdown-editor',
                template: "<div class=\"maddob-markdown-editor-container\" [ngClass]=\"{'fullscreen': fullscreenEnabled}\">\n  \n    <!-- HEADER -->\n    <div class=\"maddob-markdown-editor-header\">\n  \n      <div class=\"maddob-markdown-editor-left\">\n        <button [disabled]=\"disabled\" (click)=\"insertContent('bold')\">bold</button>\n        <button [disabled]=\"disabled\" (click)=\"insertContent('italic')\">italic</button>\n        <button [disabled]=\"disabled\" (click)=\"insertContent('quote')\">quote</button>\n        <button [disabled]=\"disabled\" (click)=\"insertContent('code')\">code</button>\n      </div>\n  \n      <div class=\"maddob-markdown-editor-center\">\n          <button [disabled]=\"disabled\"  (click)=\"insertContent('list')\">list</button>\n          <button [disabled]=\"disabled\"  (click)=\"insertContent('link')\">link</button>\n          <button [disabled]=\"disabled\"  (click)=\"insertContent('image')\">image</button>\n        </div>\n  \n      <div class=\"maddob-markdown-editor-right\">\n        <button (click)=\"togglePreview()\">preview on</button>\n        <button (click)=\"toggleFullscreen()\">fullscreen</button>\n      </div>\n      \n    </div>\n  \n    <!-- CONTENT -->\n    <div class=\"maddob-markdown-editor-content\">\n  \n      <div class=\"maddob-markdown-editor\" (window:resize)=\"resizeCodeMirror()\"\n        [ngClass]=\"{'default-height': !fullscreenEnabled, 'full-height': fullscreenEnabled}\">\n          <textarea #editor></textarea>\n      </div>\n  \n      <div class=\"maddob-markdown-preview\" [innerHTML]=\"parsedHtml\" *ngIf=\"previewEnabled\"\n        [ngClass]=\"{'default-height': !fullscreenEnabled, 'full-height': fullscreenEnabled}\">\n      </div>\n  \n    </div>\n  </div>\n  ",
                providers: [
                    {
                        provide: NG_VALUE_ACCESSOR,
                        multi: true,
                        useExisting: forwardRef((/**
                         * @return {?}
                         */
                        () => MarkdownEditorComponent)),
                    },
                    {
                        provide: NG_VALIDATORS,
                        multi: true,
                        useExisting: forwardRef((/**
                         * @return {?}
                         */
                        () => MarkdownEditorComponent))
                    }
                ],
                styles: [".maddob-markdown-editor-container{display:flex;flex-direction:column;border:1px solid #000}.maddob-markdown-editor-header{display:flex;flex-direction:row;justify-content:space-between;padding:5px}.maddob-markdown-editor-header-left{display:flex;justify-content:flex-start}.maddob-markdown-editor-header-center{flex-grow:0;display:flex;justify-content:center}.maddob-markdown-editor-header-right{display:flex;justify-content:flex-end}.maddob-markdown-editor-content{display:flex;flex-grow:1;flex-direction:row;border-top:1px solid #000}.maddob-markdown-editor{flex-basis:50%;flex-grow:1;text-align:left;overflow:scroll}.maddob-markdown-editor textarea{width:100%;height:100%}.maddob-markdown-preview{text-align:left;flex-basis:50%;border-left:1px solid #ccc;overflow:scroll;padding:5px}.default-height{height:300px}.full-height{height:100%}.fullscreen{background-color:#fff;height:100%;width:100%;left:0;top:0;overflow:hidden;position:fixed;z-index:1000}"]
            }] }
];
/** @nocollapse */
MarkdownEditorComponent.ctorParameters = () => [
    { type: Boolean, decorators: [{ type: Attribute, args: ['required',] }] },
    { type: Number, decorators: [{ type: Attribute, args: ['maxlength',] }] }
];
MarkdownEditorComponent.propDecorators = {
    editor: [{ type: ViewChild, args: ["editor",] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class UikitMarkdownEditorComponent extends MarkdownEditorComponent {
    /**
     * @return {?}
     */
    ngOnInit() {
    }
    /**
     * @return {?}
     */
    resizeCodeMirror() {
        setTimeout((/**
         * @return {?}
         */
        () => {
            /** @type {?} */
            const editorContainer = this.editor.nativeElement.parentNode;
            this.codeMirror.setSize(editorContainer.offsetWidth - 50, editorContainer.offsetHeight);
        }), 200);
    }
}
UikitMarkdownEditorComponent.decorators = [
    { type: Component, args: [{
                selector: 'maddob-uikit-markdown-editor',
                template: "<div [ngClass]=\"{'fullscreen': fullscreenEnabled}\" class=\"full-border\">\n  <div class=\"CodeMirror-Editor-Tab uk-background-muted uk-flex\">\n    <div class=\"uk-flex uk-flex-left\">\n      <a uk-icon=\"icon: bold\" (click)=\"insertContent('bold')\" uk-tooltip=\"title: Bold; delay: 800\"></a>\n      <a uk-icon=\"icon: italic\" (click)=\"insertContent('italic')\" uk-tooltip=\"title: Italic; delay: 800\"></a>\n      <a uk-icon=\"icon: quote-right\" (click)=\"insertContent('quote')\" uk-tooltip=\"title: Blockquote; delay: 800\"></a>\n    </div>\n\n    <div class=\"uk-flex uk-flex-left uk-margin-left separator\"></div>\n\n    <div class=\"uk-flex uk-flex-left uk-margin-left\">\n      <a uk-icon=\"icon: list\" (click)=\"insertContent('list')\" uk-tooltip=\"title: List; delay: 800\"></a>\n      <a uk-icon=\"icon: code\" (click)=\"insertContent('code')\" uk-tooltip=\"title: Code; delay: 800\"></a>\n    </div>\n\n    <div class=\"uk-flex uk-flex-left uk-margin-left separator\"></div>\n\n    <div class=\"uk-flex uk-flex-left uk-margin-left\">\n      <a uk-icon=\"icon: link\" (click)=\"insertContent('link')\" uk-tooltip=\"title: Link; delay: 800\"></a>\n      <a uk-icon=\"icon: image\" (click)=\"insertContent('image')\" uk-tooltip=\"title: image; delay: 800\"></a>\n    </div>\n\n    <div class=\"uk-flex uk-flex-left uk-margin-left separator\"></div>\n\n    <div class=\"uk-flex uk-flex-left uk-margin-left\">\n      <a *ngIf=\"previewEnabled\" uk-icon=\"icon: chevron-right\" (click)=\"previewEnabled = false; resizeCodeMirror()\"></a>\n      <a *ngIf=\"!previewEnabled\" uk-icon=\"icon: chevron-left\" (click)=\"previewEnabled = true; resizeCodeMirror()\"></a>\n    </div>\n\n    <div class=\"uk-flex uk-flex-right\" style=\"flex-grow: 1;\">\n      <a *ngIf=\"!fullscreenEnabled\" uk-icon=\"icon: expand\" (click)=\"toggleFullscreen()\"></a>\n      <a *ngIf=\"fullscreenEnabled\" uk-icon=\"icon: shrink\" (click)=\"toggleFullscreen()\"></a>\n    </div>\n  </div>\n\n\n  <div class=\"editor-content\" class=\"uk-grid\" uk-grid\n    [ngClass]=\"{'max-height': !fullscreenEnabled, 'full-height': fullscreenEnabled}\">\n    <div class=\"code-mirror-container\"\n      [ngClass]=\"{'uk-width-1-2': previewEnabled, 'uk-width-expand': !previewEnabled, 'max-height': !fullscreenEnabled}\"\n      #codeMirrorContainer (window:resize)=\"resizeCodeMirror()\">\n      <textarea #editor name=\"content\"></textarea>\n    </div>\n    <div *ngIf=\"previewEnabled\" class=\"preview\" [ngClass]=\"{'uk-width-1-2': previewEnabled, 'hidden': !previewEnabled }\"\n      [innerHTML]=\"parsedHtml\"></div>\n  </div>\n</div>",
                providers: [
                    {
                        provide: NG_VALUE_ACCESSOR,
                        multi: true,
                        useExisting: forwardRef((/**
                         * @return {?}
                         */
                        () => UikitMarkdownEditorComponent)),
                    },
                    {
                        provide: NG_VALIDATORS,
                        multi: true,
                        useExisting: forwardRef((/**
                         * @return {?}
                         */
                        () => UikitMarkdownEditorComponent))
                    }
                ],
                styles: [".full-border,.html-editor-container{border:1px solid #e5e5e5}.CodeMirror-Editor-Tab{padding:10px;border-bottom:1px solid #e5e5e5}.editor-content{border:1px solid #e5e5e5}.fullscreen{height:100%;width:100%;left:0;top:0;overflow:hidden;position:fixed;z-index:1000}.preview{overflow:scroll;overflow-wrap:break-word;border-left:1px solid #e5e5e5;height:300px;padding:5px}.hidden{display:none}.max-height{height:300px;overflow:scroll}.full-height,.full-height textarea,.full-height>.uk-width-1-2>.CodeMirror{height:100%}.full-height div{height:100%;background-color:#fff;display:block}#preview-content{background:#fff;padding:5px}.uk-icon{padding:0 5px}.separator{width:1px;background-color:#e5e5e5}"]
            }] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class MaddobLibModule {
}
MaddobLibModule.decorators = [
    { type: NgModule, args: [{
                declarations: [MarkdownEditorComponent, UikitMarkdownEditorComponent],
                imports: [
                    CommonModule,
                    FormsModule
                ],
                exports: [MarkdownEditorComponent, UikitMarkdownEditorComponent]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { MarkdownEditorComponent, UikitMarkdownEditorComponent, MaddobLibModule };

//# sourceMappingURL=maddob-lib.js.map