import { OnInit, AfterViewInit, ElementRef } from '@angular/core';
import { ControlValueAccessor, Validator, AbstractControl, ValidationErrors } from '@angular/forms';
export declare class MarkdownEditorComponent implements OnInit, AfterViewInit, ControlValueAccessor, Validator {
    required: boolean;
    maxlength: number;
    /** CodeMirror editor instance */
    protected codeMirror: any;
    /** editor text value */
    private _value;
    /** parsed markdown value will be stored here */
    parsedHtml: string;
    /** flag for fullscreen mode */
    fullscreenEnabled: boolean;
    /** flag for show/hide preview area */
    previewEnabled: boolean;
    /** flag for disabling the editor (and all its controls) */
    disabled: boolean;
    value: string;
    onChange: (newValue: string) => void;
    onTouched: () => void;
    editor: ElementRef;
    constructor(required?: boolean, maxlength?: number);
    ngOnInit(): void;
    ngAfterViewInit(): void;
    writeValue(obj: any): void;
    registerOnChange(fn: (value: string) => void): void;
    registerOnTouched(fn: any): void;
    setDisabledState?(isDisabled: boolean): void;
    /**
     * Validate implementation
     *
     * only required and maxlength for now
     */
    validate(control: AbstractControl): ValidationErrors;
    /**
     * Resizes the code mirror instance to the size of the container
     *
     * Used when the code mirror editor element size is changed (window resize, )
     */
    resizeCodeMirror(): void;
    toggleFullscreen(): void;
    togglePreview(): void;
    /**
     * Inserts special content into the editor
     *
     * can be used for different buttons like bold, italic,
     *
     * Override the method if you want to customize the behaviour of the
     * insertion. For example if you have an own image selector and so on.
     *
     * @param type type of the content that is to be inserted
     * @param value optional value (for special custom content)
     */
    insertContent(type: string, value?: string): void;
}
